import { NgModule, Component, EventEmitter, Input, OnInit, Output, OnChanges, SimpleChanges } from '@angular/core';
import { CommonModule } from '@angular/common';
import { FormsModule } from '@angular/forms';
import { CatalogService } from '../kernel/services/catalog.service';
import { FieldService } from '../kernel/services/field.service';

import { LoadingBannerModule } from '../loadingBanner';
import { DatePickerModule } from '../datePicker';

import { DropdownModule } from 'primeng/components/dropdown/dropdown';
import { TooltipModule } from 'primeng/components/tooltip/tooltip';

@Component({
    selector: 'glx-dynamic-field',
    styleUrls: ['dynamicField.scss'],
    templateUrl: 'dynamicField.html'
})

export class DynamicField implements OnInit, OnChanges {

    @Input() model: any;
    @Input() category_id: any = '';
    @Input() loading: boolean = false;
    @Input() grid: string = 'col-12 col-lg-6';
    @Input() theme: string = 'default';
    @Input() secure: boolean = true;
    @Input() readonly: boolean = false;
    @Input() fieldUrl: string = '';
    @Input() catalogUrl: string = '';


    @Input() fields: any[] = [];
    @Input() catalogs: any = {};

    @Output() modelExport: EventEmitter<any> = new EventEmitter<any>();

    _fields: any = [];
    _catalogs: any = {};
    _valid: any = {};
    classes: string = '';

    constructor(private fieldService: FieldService,
                private catalogService: CatalogService) {
    }

    ngOnInit() {
        /**
         *  Если есть ID, то получаем поля и справочники самостоятельно, иначе берем из входных параметров
         */
        if (this.category_id && this.category_id.length > 0) {
            if (this.fieldUrl.length > 0) {
                this.fieldService.setUrl(this.fieldUrl);
            }
            if (this.catalogUrl.length > 0) {
                this.catalogService.setUrl(this.catalogUrl);
            }
            this.getFields(this.category_id);
        } else {
            if (this.fields && this.fields.length > 0) {
                this._fields = this.fields;
            }
            if (this.catalogs && this.catalogs.length > 0) {
                this._catalogs = this.catalogs;
            }
        }

        this.classes = this.theme + ' ' + this.grid;
    }

    /*
     Отслеживаем изменения входных параметров
     */
    ngOnChanges(changes: SimpleChanges) {
        if (changes.category_id) {
            this.clearFields();
            this.getFields(this.category_id);
        }
        if (changes.fields) {
            this.clearFields();
            this._fields = this.fields;
        }
        if (changes.catalogs) {
            this._catalogs = this.catalogs;
        }
        if (changes.fieldUrl && this.fieldUrl.length > 0) {
            this.fieldService.setUrl(this.fieldUrl);
        }
        if (changes.catalogUrl && this.catalogUrl.length > 0) {
            this.catalogService.setUrl(this.catalogUrl);
        }

    }

    /*
     Очищаем все Доп.поля (например, при смене категории)
     */
    private clearFields() {
        for (let i = 0; i < this._fields.length; i++) {
            delete this.model[this._fields[i].name];
        }
        this._valid = {};
        this.modelExport.emit(this.model);
    }

    update(field?) {
        if (field) {
            this._valid[field.name] = {
                valid: field.valid,
                invalid: !field.valid,
                dirty: field.dirty,
                touched: field.touched,
            };
        }
        this.modelExport.emit(this.model);
    }

    valid(field) {
        if (this._valid[field]) {
            return this._valid[field].valid && (this._valid[field].dirty || this._valid[field].touched);
        } else {
            return false;
        }
    }

    invalid(field) {
        if (this._valid[field]) {
            return this._valid[field].invalid && (this._valid[field].dirty || this._valid[field].touched);
        } else {
            return false;
        }
    }

    /*
     Получить доп.поля по категории
     */
    private getFields(id: number) {
        if (id > 0) {
            this.loading = true;
            this._fields = [];
            this.fieldService.get(id, this.secure)
                .subscribe(
                    result => {
                        for (let i = 0; i < result.length; i++) {
                            result[i].required = result.required || result[i].required === 1 || result[i].required === '1';
                            if (result.visible || result[i].visible === 1 || result[i].visible === '1') {
                                this._fields.push(result[i]);
                            }
                        }
                        this.getCatalogs();
                        this.loading = false;
                    },
                    error => {
                        this.loading = false;
                        console.error(error);
                    });
        }
    }

    /*
     Получить каталоги для доп.полей
     */
    private getCatalogs() {
        this._catalogs = {};
        for (let i = 0; i < this._fields.length; i++) {
            if (this._fields[i].type === 'catalog') {
                if (this._fields[i].list.length > 0) {
                    this.getCatalog(this._fields[i].list);
                } else {
                    console.error('Неопределен справочник для поля ' + this._fields[i].title + '(' + this._fields[i].name + ')');
                }
            }
        }
    }

    /*
     Получить каталог для доп.полей
     */
    private getCatalog(catalog ?: string) {
        this.catalogService.get(catalog, this.secure)
            .subscribe(
                result => {
                    this._catalogs[catalog] = result;
                },
                error => {
                    console.error(error);
                });
    }
}


@NgModule({
    imports: [CommonModule, FormsModule, LoadingBannerModule, DatePickerModule, DropdownModule, TooltipModule],
    exports: [DynamicField],
    declarations: [DynamicField],
    providers: [
        FieldService,
        CatalogService
    ]
})

export class DynamicFieldModule {
}
