import { Component, Input, NgModule, OnInit } from '@angular/core';
import { CommonModule } from '@angular/common';
import { TabViewModule } from 'primeng/components/tabview/tabview';
import { AccordionModule } from 'primeng/components/accordion/accordion';
import { TooltipModule } from 'primeng/components/tooltip/tooltip';


import { DefaultOptions } from './defaultOptions';

/**
 * Зачетная книжка обучающегося
 *
 * Показывает оценки обучающегося с разбивкой по годам и семестрам.
 *
 * Пример:
 * <example-url>http://localhost:4220/recordBookViewer</example-url>
 *
 * @example
 * <glx-record-book-viewer [data]="marks" [options]="options"></glx-record-book-viewer>
 */
@Component({
    selector: 'glx-record-book-viewer',
    styleUrls: ['./recordBookViewer.scss'],
    templateUrl: './recordBookViewer.html'
})

export class RecordBookViewer implements OnInit {
    /**
     * Массив оценок
     * @type {Array}
     */
    @Input() data: any[] = [];
    /**
     * Разрешить множественное открытие гармошки
     * @type {boolean}
     */
    @Input() multiple = true;
    /**
     * Настройки
     * @type any
     */
    @Input() options: any = DefaultOptions;

    /**
     * Колонки для таблицы
     * @type {Array}
     * @private
     */
    _columns: any = [];

    /**
     * Конструктор
     */
    constructor() {
    }

    /**
     * Инициализация
     */
    ngOnInit() {
        this.initColumns();
    }

    /**
     * Формирует массив колонок, которые нужно показывать и сортирует его
     */
    initColumns() {
        for (let i = 0; i < this.options.columns.length; i++) {
            if (this.options.columns[i].show &&
                (this.options.columns[i].show === 1 || this.options.columns[i].show === true)) {
                this._columns.push(this.options.columns[i]);
            }
        }

        this._columns.sort(function (a, b) {
            return (a.sort > b.sort) ? 1 : ((b.sort > a.sort) ? -1 : 0);
        });
    }

    /**
     * Формирует название вкладки
     *
     * @param year Год
     *
     * @example
     * formatYear(2017) // =\> 2017 / 2018
     *
     * @returns {string} Год / Следующий год
     */
    formatYear(year) {
        return year + ' / ' + (year + 1);
    }

    /**
     * Получить класс оценки.
     * Используется для отображения кубка.
     * @param {string} mark_title Текст с названием оценки
     *
     * @example
     * getMarkClass(\"отлично\") // =\> mark-5
     *
     * @returns {any}
     */
    getMarkClass(mark_title: string) {
        switch (mark_title.toLowerCase()) {
            case 'отлично':
                return 'mark-5';
            case 'хорошо':
                return 'mark-4';
            default:
                return 'mark-none';
        }
    }
}

@NgModule({
    imports: [CommonModule, AccordionModule, TabViewModule, TooltipModule],
    exports: [RecordBookViewer],
    declarations: [RecordBookViewer]
})

export class RecordBookViewerModule {
}
