import { Component, EventEmitter, Input, NgModule, OnInit, Output, ViewChild } from '@angular/core';
import { FormsModule } from '@angular/forms';
import { CommonModule } from '@angular/common';
import { ActivatedRoute, Router, RouterModule } from '@angular/router';

import { DataTable, DataTableModule } from 'primeng/components/datatable/datatable';
import { DropdownModule } from 'primeng/components/dropdown/dropdown';
import { TooltipModule } from 'primeng/components/tooltip/tooltip';
import { SharedModule } from 'primeng/components/common/shared';

import { LoadingBannerModule } from '../loadingBanner/loadingBanner';

import { GlxSharedModule } from '../kernel/shared.module';

/**
 * Фильтр по сотрудникам
 */
class Filter {
    global: string;

    constructor() {

    }
}

/**
 * Поиск сотрудников с детальным просмотром профиля
 */
@Component({
    selector: 'glx-user-work-search',
    styleUrls: ['./userWorkSearch.scss'],
    templateUrl: './userWorkSearch.html'
})

export class UserWorkSearch implements OnInit {
    /**
     * Данные для отображения в таблице
     */
    @Input() data: any = {};
    /**
     * Справочники для фильтров
     * @type {{}}
     */
    @Input() catalog: any = {};
    /**
     * Открыт ли фильтр
     * @type {boolean}
     */
    @Input() filterShow: boolean = false;
    /**
     * Показать индикатор загрузки данных
     * @type {boolean}
     */
    @Input() loading: boolean = false;
    /**
     * Признак того, что компонент открыт в слайдере
     * @type {boolean}
     */
    @Input() slider: boolean = false;
    /**
     * Изображение профиля по-умолчанию
     * @type {string}
     */
    @Input() defaultImage: string = '';
    /**
     * Задержка поиска для общего фильтра
     * @type {number}
     */
    @Input() filterDelay: number = 1000;
    /**
     * Фильтр
     * @type {Filter}
     */
    @Input() filter: Filter = new Filter();
    /**
     * Надпись на глобальном фильтре
     * @type {string}
     */
    @Input() globalFilterPlaceholder: string = 'Поиск...';
    /**
     * Событие фильтрации. Необходимо получить новые данные сервера
     * @type {EventEmitter<any>}
     */
    @Output() onFilter: EventEmitter<any> = new EventEmitter();
    /**
     * Событий просмотра профиля. Необходимо получить профиль пользователя с сервера
     * @type {EventEmitter<any>}
     */
    @Output() onViewUser: EventEmitter<any> = new EventEmitter();

    /**
     * Поле для предфильтрации
     */
    @Input() filterField: any;
    /**
     * Значение поля для предфильтрации
     */
    @Input() filterValue: any;
    /**
     * Префикс пути для изображений
     */
    @Input() imagePrefix: string;

    /**
     * PrimeNG DataTable
     */
    @ViewChild('dt') datatable: DataTable;
    /**
     * PrimeNg Global Search Field
     */
    @ViewChild('gb') gb: any;
    /**
     * Timeout экземпляр фильтрации
     */
    filterTimeout: any;
    /**
     * Первое срабатывание фильтрации
     * @type {boolean}
     */
    firstFilter: boolean = true;

    /**
     * Конструктор
     * @param {Router} router
     * @param {ActivatedRoute} route
     * @param {Renderer2} renderer
     */
    constructor(private router: Router,
                private route: ActivatedRoute) {
    }

    /**
     * Проверка префильтров
     */
    ngOnInit() {
        this.route.paramMap.subscribe(
            params => {
                if (!this.slider) {
                    this.filterField = params.get('field');
                    this.filterValue = params.get('value');
                    if (!this.firstFilter) {
                        this.checkPreFilter();
                        this.datatable._filter();
                    }
                }
            }
        );
    }

    /**
     * Получить путь к фото
     * @param photo
     * @returns {any}
     */
    getPhoto(photo) {
        if (!photo) {
            return null;
        }
        return this.imagePrefix + photo;
    }

    /**
     * Выключить поле если по нему есть префильтрация
     * @param field
     * @returns {boolean}
     */
    ifDisabled(field) {
        return !!(this.filterField && this.filterValue && this.filterField === field);
    }

    /**
     * Проверка префильтра
     */
    checkPreFilter() {
        if (this.route.snapshot.paramMap.get('field') && this.route.snapshot.paramMap.get('value') && this.firstFilter) {
            this.filterField = this.route.snapshot.paramMap.get('field');
            this.filterValue = this.route.snapshot.paramMap.get('value');
        }

        if (this.filterField && this.filterValue) {
            this.filter[this.filterField] = this.filterValue;
            if (this.filterValue === parseInt(this.filterValue, 10)) {
                this.datatable.filters[this.filterField] = {
                    value: parseInt(this.filterValue, 10),
                    matchMode: 'any'
                };
            } else {
                this.datatable.filters[this.filterField] = {
                    value: this.filterValue,
                    matchMode: 'any'
                };
            }
        }
    }

    /**
     * Очистить фильтр
     */
    clearFilter() {
        if (this.filterField || this.filterValue) {
            this.filterField = null;
            this.filterValue = null;

            if (!this.slider) {
                this.router.navigate(['/search/user/work']);
            }
        }

        this.filter = new Filter();
        if (this.datatable.globalFilter && this.datatable.globalFilter.value) {
            this.datatable.globalFilter.value = '';
        }
        this.datatable.reset();
        this.datatable.sortField = 'fullname';
        this.datatable._filter();
    }

    /**
     * Скрыть / показать фильтры
     */
    toggleFilter() {
        this.filterShow = !this.filterShow;
    }

    /**
     * Применение глобального фильтра
     * @param gbt
     */
    globalFilterApply(gbt) {
        if (this.filterTimeout) {
            clearTimeout(this.filterTimeout);
        }
        if (gbt && gbt.value && gbt.value.length > 2) {
            this.filterTimeout = setTimeout(() => {
                this.datatable.globalFilter = gbt;
                this.datatable._filter();
                this.filterTimeout = null;
            }, this.filterDelay);
        }
        if (gbt && gbt.value === '') {
            this.datatable.globalFilter.value = '';
            this.datatable._filter();
        }
    }

    /**
     * Установка и применение фильтра
     * @param name
     * @param value
     */
    setFilter(name, value) {
        switch (name) {
            case 'fullname':
                if (this.filterTimeout) {
                    clearTimeout(this.filterTimeout);
                }
                if (value && value.length > 2) {
                    this.filterTimeout = setTimeout(() => {
                        this.datatable.filter(value, name, 'any');
                        this.filterTimeout = null;
                    }, this.filterDelay);
                }
                if (value === '') {
                    this.datatable.filter(value, name, 'any');
                }
                break;
            default:
                this.datatable.filter(value, name, 'any');
        }

    }

    /**
     * Устанавилвает фильтр из контекста (при клике на значение в таблице)
     * @param name string Название поля для фильтрации
     * @param value string Значение поля
     */
    setContextFilter(name, value) {
        this.filter = new Filter();
        this.datatable.filters = {};
        this.datatable.filteredValue = null;
        this.datatable.sortField = 'fullname';
        this.filter[name] = value;
        this.setFilter(name, value);
    }

    /**
     * Событие для ленивой загрузки. Вызывается при инициализации и фильтрации
     * @param event
     */
    loadItemsLazy(event: any) {
        if (this.firstFilter) {
            this.checkPreFilter();
            this.firstFilter = false;
        }

        this.onFilter.emit(event);
    }

    /**
     * Собатие показа профиля в слайдере
     * @param id
     */
    showUser(id: number) {
        this.onViewUser.emit({id: id});
    }

    /**
     * Одиночный щелчок по строке устанавливает ID профиля для возможного открытия
     * @param event
     */
    onRowClick(event) {
        this.showUser(event.data.id);
    }
}

@NgModule({
    imports: [
        CommonModule,
        FormsModule,
        RouterModule,
        SharedModule,
        GlxSharedModule,
        DataTableModule,
        DropdownModule,
        TooltipModule,
        LoadingBannerModule
    ],
    exports: [UserWorkSearch],
    declarations: [UserWorkSearch]
})

export class UserWorkSearchModule {
}
